@@ -5,7 +5,7 @@
 // This file is licensed under the GPLv2 or later
 //
 // Copyright 2006-2008 Julien Rebetez <julienr@svn.gnome.org>
-// Copyright 2007 Albert Astals Cid <aacid@kde.org>
+// Copyright 2007-2009 Albert Astals Cid <aacid@kde.org>
 // Copyright 2007-2008 Carlos Garcia Campos <carlosgc@gnome.org>
 // Copyright 2007 Adrian Johnson <ajohnson@redneon.com>
 // Copyright 2007 Iñigo Martínez <inigomartinez@gmail.com>
@@ -258,11 +258,13 @@ void FormWidgetButton::loadDefaults ()
 	      break;
           }
         } else if (obj2.isStream()) {
+          // TODO do something with str and obj3
           Stream *str = obj2.getStream();
           Dict *tmpDict2 = str->getDict();
           Object obj3;
           tmpDict2->lookup("Length", &obj3);
           onStr = new GooString ("D");
+          obj3.free();
         }
         obj2.free();
 	if (onStr)
