@@ -1,5 +1,6 @@
 /* poppler-annotation.cc: qt interface to poppler
- * Copyright (C) 2006, Albert Astals Cid
+ * Copyright (C) 2006, Albert Astals Cid <aacid@kde.org>
+ * Copyright (C) 2006, 2008 Pino Toscano <pino@kde.org>
  * Adapting code from
  *   Copyright (C) 2004 by Enrico Ros <eros.kde@email.it>
  *
@@ -25,8 +26,13 @@
 // local includes
 #include "poppler-annotation.h"
 #include "poppler-link.h"
+#include "poppler-qt4.h"
 #include "poppler-annotation-private.h"
 
+// poppler includes
+#include <Page.h>
+#include <Annot.h>
+
 namespace Poppler {
 
 //BEGIN AnnotationUtils implementation
@@ -59,6 +65,9 @@ Annotation * AnnotationUtils::createAnnotation( const QDomElement & annElement )
         case Annotation::AInk:
             annotation = new InkAnnotation( annElement );
             break;
+        case Annotation::ACaret:
+            annotation = new CaretAnnotation( annElement );
+            break;
     }
 
     // return created annotation
@@ -669,19 +678,19 @@ class LineAnnotationPrivate : public AnnotationPrivate
         QLinkedList<QPointF> linePoints;
         LineAnnotation::TermStyle lineStartStyle;
         LineAnnotation::TermStyle lineEndStyle;
-        bool lineClosed;  // (if true draw close shape)
+        bool lineClosed : 1;  // (if true draw close shape)
+        bool lineShowCaption : 1;
         QColor lineInnerColor;
         double lineLeadingFwdPt;
         double lineLeadingBackPt;
-        bool lineShowCaption;
         LineAnnotation::LineIntent lineIntent;
 };
 
 LineAnnotationPrivate::LineAnnotationPrivate()
     : AnnotationPrivate(), lineStartStyle( LineAnnotation::None ),
     lineEndStyle( LineAnnotation::None ), lineClosed( false ),
-    lineLeadingFwdPt( 0 ), lineLeadingBackPt( 0 ),
-    lineShowCaption( false ), lineIntent( LineAnnotation::Unknown )
+    lineShowCaption( false ), lineLeadingFwdPt( 0 ), lineLeadingBackPt( 0 ),
+    lineIntent( LineAnnotation::Unknown )
 {
 }
 
@@ -1443,6 +1452,7 @@ LinkAnnotation::LinkAnnotation( const QDomNode & node )
                 {
                     Poppler::LinkAction::ActionType act;
                     QString actString = qe.attribute( "action" );
+                    bool found = true;
                     if ( actString == "PageFirst" )
                         act = Poppler::LinkAction::PageFirst;
                     else if ( actString == "PagePrev" )
@@ -1467,14 +1477,21 @@ LinkAnnotation::LinkAnnotation( const QDomNode & node )
                         act = Poppler::LinkAction::GoToPage;
                     else if ( actString == "Close" )
                         act = Poppler::LinkAction::Close;
-                    Poppler::LinkAction * action = new Poppler::LinkAction( QRect(), act );
-                    d->linkDestination = action;
+                    else
+                        found = false;
+                    if (found)
+                    {
+                        Poppler::LinkAction * action = new Poppler::LinkAction( QRect(), act );
+                        d->linkDestination = action;
+                    }
                 }
+#if 0
                 else if ( type == "Movie" )
                 {
                     Poppler::LinkMovie * movie = new Poppler::LinkMovie( QRect() );
                     d->linkDestination = movie;
                 }
+#endif
             }
         }
 
@@ -1591,10 +1608,14 @@ void LinkAnnotation::store( QDomNode & node, QDomDocument & document ) const
             }
             case Poppler::Link::Movie:
             {
-                Poppler::LinkMovie * movie = static_cast< Poppler::LinkMovie * >( d->linkDestination );
                 hyperlinkElement.setAttribute( "type", "Movie" );
                 break;
             }
+            case Poppler::Link::Sound:
+            {
+                // FIXME: implement me
+                break;
+            }
             case Poppler::Link::None:
                 break;
         }
@@ -1649,4 +1670,393 @@ void LinkAnnotation::setLinkRegionPoint( int id, const QPointF &point )
     d->linkRegion[id] = point;
 }
 
+/** CaretAnnotation [Annotation] */
+class CaretAnnotationPrivate : public AnnotationPrivate
+{
+    public:
+        CaretAnnotationPrivate();
+
+        // data fields
+        CaretAnnotation::CaretSymbol symbol;
+};
+
+static QString caretSymbolToString( CaretAnnotation::CaretSymbol symbol )
+{
+    switch ( symbol )
+    {
+        case CaretAnnotation::None:
+            return QString::fromLatin1( "None" );
+        case CaretAnnotation::P:
+            return QString::fromLatin1( "P" );
+    }
+    return QString();
+}
+
+static CaretAnnotation::CaretSymbol caretSymbolFromString( const QString &symbol )
+{
+    if ( symbol == QLatin1String( "None" ) )
+        return CaretAnnotation::None;
+    else if ( symbol == QLatin1String( "P" ) )
+        return CaretAnnotation::P;
+    return CaretAnnotation::None;
+}
+
+CaretAnnotationPrivate::CaretAnnotationPrivate()
+    : AnnotationPrivate(), symbol( CaretAnnotation::None )
+{
+}
+
+CaretAnnotation::CaretAnnotation()
+    : Annotation( *new CaretAnnotationPrivate() )
+{
+}
+
+CaretAnnotation::CaretAnnotation( const QDomNode & node )
+    : Annotation( *new CaretAnnotationPrivate(), node )
+{
+    Q_D( CaretAnnotation );
+
+    // loop through the whole children looking for a 'caret' element
+    QDomNode subNode = node.firstChild();
+    while( subNode.isElement() )
+    {
+        QDomElement e = subNode.toElement();
+        subNode = subNode.nextSibling();
+        if ( e.tagName() != "caret" )
+            continue;
+
+        // parse the attributes
+        if ( e.hasAttribute( "symbol" ) )
+            d->symbol = caretSymbolFromString( e.attribute( "symbol" ) );
+
+        // loading complete
+        break;
+    }
+}
+
+CaretAnnotation::~CaretAnnotation()
+{
+}
+
+void CaretAnnotation::store( QDomNode & node, QDomDocument & document ) const
+{
+    Q_D( const CaretAnnotation );
+
+    // recurse to parent objects storing properties
+    Annotation::store( node, document );
+
+    // create [caret] element
+    QDomElement caretElement = document.createElement( "caret" );
+    node.appendChild( caretElement );
+
+    // append the optional attributes
+    if ( d->symbol != CaretAnnotation::None )
+        caretElement.setAttribute( "symbol", caretSymbolToString( d->symbol ) );
+}
+
+Annotation::SubType CaretAnnotation::subType() const
+{
+    return ACaret;
+}
+
+CaretAnnotation::CaretSymbol CaretAnnotation::caretSymbol() const
+{
+    Q_D( const CaretAnnotation );
+    return d->symbol;
+}
+
+void CaretAnnotation::setCaretSymbol( CaretAnnotation::CaretSymbol symbol )
+{
+    Q_D( CaretAnnotation );
+    d->symbol = symbol;
+}
+
+/** FileAttachmentAnnotation [Annotation] */
+class FileAttachmentAnnotationPrivate : public AnnotationPrivate
+{
+    public:
+        FileAttachmentAnnotationPrivate();
+        ~FileAttachmentAnnotationPrivate();
+
+        // data fields
+        QString icon;
+        EmbeddedFile *embfile;
+};
+
+FileAttachmentAnnotationPrivate::FileAttachmentAnnotationPrivate()
+    : AnnotationPrivate(), icon( "PushPin" ), embfile( 0 )
+{
+}
+
+FileAttachmentAnnotationPrivate::~FileAttachmentAnnotationPrivate()
+{
+    delete embfile;
+}
+
+FileAttachmentAnnotation::FileAttachmentAnnotation()
+    : Annotation( *new FileAttachmentAnnotationPrivate() )
+{
+}
+
+FileAttachmentAnnotation::FileAttachmentAnnotation( const QDomNode & node )
+    : Annotation( *new FileAttachmentAnnotationPrivate(), node )
+{
+    // loop through the whole children looking for a 'fileattachment' element
+    QDomNode subNode = node.firstChild();
+    while( subNode.isElement() )
+    {
+        QDomElement e = subNode.toElement();
+        subNode = subNode.nextSibling();
+        if ( e.tagName() != "fileattachment" )
+            continue;
+
+        // loading complete
+        break;
+    }
+}
+
+FileAttachmentAnnotation::~FileAttachmentAnnotation()
+{
+}
+
+void FileAttachmentAnnotation::store( QDomNode & node, QDomDocument & document ) const
+{
+    // recurse to parent objects storing properties
+    Annotation::store( node, document );
+
+    // create [fileattachment] element
+    QDomElement fileAttachmentElement = document.createElement( "fileattachment" );
+    node.appendChild( fileAttachmentElement );
+}
+
+Annotation::SubType FileAttachmentAnnotation::subType() const
+{
+    return AFileAttachment;
+}
+
+QString FileAttachmentAnnotation::fileIconName() const
+{
+    Q_D( const FileAttachmentAnnotation );
+    return d->icon;
+}
+
+void FileAttachmentAnnotation::setFileIconName( const QString &icon )
+{
+    Q_D( FileAttachmentAnnotation );
+    d->icon = icon;
+}
+
+EmbeddedFile* FileAttachmentAnnotation::embeddedFile() const
+{
+    Q_D( const FileAttachmentAnnotation );
+    return d->embfile;
+}
+
+void FileAttachmentAnnotation::setEmbeddedFile( EmbeddedFile *ef )
+{
+    Q_D( FileAttachmentAnnotation );
+    d->embfile = ef;
+}
+
+/** SoundAnnotation [Annotation] */
+class SoundAnnotationPrivate : public AnnotationPrivate
+{
+    public:
+        SoundAnnotationPrivate();
+        ~SoundAnnotationPrivate();
+
+        // data fields
+        QString icon;
+        SoundObject *sound;
+};
+
+SoundAnnotationPrivate::SoundAnnotationPrivate()
+    : AnnotationPrivate(), icon( "Speaker" ), sound( 0 )
+{
+}
+
+SoundAnnotationPrivate::~SoundAnnotationPrivate()
+{
+    delete sound;
+}
+
+SoundAnnotation::SoundAnnotation()
+    : Annotation( *new SoundAnnotationPrivate() )
+{
+}
+
+SoundAnnotation::SoundAnnotation( const QDomNode & node )
+    : Annotation( *new SoundAnnotationPrivate(), node )
+{
+    // loop through the whole children looking for a 'sound' element
+    QDomNode subNode = node.firstChild();
+    while( subNode.isElement() )
+    {
+        QDomElement e = subNode.toElement();
+        subNode = subNode.nextSibling();
+        if ( e.tagName() != "sound" )
+            continue;
+
+        // loading complete
+        break;
+    }
+}
+
+SoundAnnotation::~SoundAnnotation()
+{
+}
+
+void SoundAnnotation::store( QDomNode & node, QDomDocument & document ) const
+{
+    // recurse to parent objects storing properties
+    Annotation::store( node, document );
+
+    // create [sound] element
+    QDomElement soundElement = document.createElement( "sound" );
+    node.appendChild( soundElement );
+}
+
+Annotation::SubType SoundAnnotation::subType() const
+{
+    return ASound;
+}
+
+QString SoundAnnotation::soundIconName() const
+{
+    Q_D( const SoundAnnotation );
+    return d->icon;
+}
+
+void SoundAnnotation::setSoundIconName( const QString &icon )
+{
+    Q_D( SoundAnnotation );
+    d->icon = icon;
+}
+
+SoundObject* SoundAnnotation::sound() const
+{
+    Q_D( const SoundAnnotation );
+    return d->sound;
+}
+
+void SoundAnnotation::setSound( SoundObject *s )
+{
+    Q_D( SoundAnnotation );
+    d->sound = s;
+}
+
+/** MovieAnnotation [Annotation] */
+class MovieAnnotationPrivate : public AnnotationPrivate
+{
+    public:
+        MovieAnnotationPrivate();
+        ~MovieAnnotationPrivate();
+
+        // data fields
+        MovieObject *movie;
+        QString title;
+};
+
+MovieAnnotationPrivate::MovieAnnotationPrivate()
+    : AnnotationPrivate(), movie( 0 )
+{
+}
+
+MovieAnnotationPrivate::~MovieAnnotationPrivate()
+{
+    delete movie;
+}
+
+MovieAnnotation::MovieAnnotation()
+    : Annotation( *new MovieAnnotationPrivate() )
+{
+}
+
+MovieAnnotation::MovieAnnotation( const QDomNode & node )
+    : Annotation( *new MovieAnnotationPrivate(), node )
+{
+    // loop through the whole children looking for a 'movie' element
+    QDomNode subNode = node.firstChild();
+    while( subNode.isElement() )
+    {
+        QDomElement e = subNode.toElement();
+        subNode = subNode.nextSibling();
+        if ( e.tagName() != "movie" )
+            continue;
+
+        // loading complete
+        break;
+    }
+}
+
+MovieAnnotation::~MovieAnnotation()
+{
+}
+
+void MovieAnnotation::store( QDomNode & node, QDomDocument & document ) const
+{
+    // recurse to parent objects storing properties
+    Annotation::store( node, document );
+
+    // create [movie] element
+    QDomElement movieElement = document.createElement( "movie" );
+    node.appendChild( movieElement );
+}
+
+Annotation::SubType MovieAnnotation::subType() const
+{
+    return AMovie;
+}
+
+MovieObject* MovieAnnotation::movie() const
+{
+    Q_D( const MovieAnnotation );
+    return d->movie;
+}
+
+void MovieAnnotation::setMovie( MovieObject *movie )
+{
+    Q_D( MovieAnnotation );
+    d->movie = movie;
+}
+
+QString MovieAnnotation::movieTitle() const
+{
+    Q_D( const MovieAnnotation );
+    return d->title;
+}
+
+void MovieAnnotation::setMovieTitle( const QString &title )
+{
+    Q_D( MovieAnnotation );
+    d->title = title;
+}
+
+//BEGIN utility annotation functions
+QColor convertAnnotColor( AnnotColor *color )
+{
+    if ( !color )
+        return QColor();
+
+    QColor newcolor;
+    double *color_data = color->getValues();
+    switch ( color->getSpace() )
+    {
+        case AnnotColor::colorTransparent: // = 0,
+            newcolor = Qt::transparent;
+            break;
+        case AnnotColor::colorGray: // = 1,
+            newcolor.setRgbF( color_data[0], color_data[0], color_data[0] );
+            break;
+        case AnnotColor::colorRGB: // = 3,
+            newcolor.setRgbF( color_data[0], color_data[1], color_data[2] );
+            break;
+        case AnnotColor::colorCMYK: // = 4
+            newcolor.setCmykF( color_data[0], color_data[1], color_data[2], color_data[3] );
+            break;
+    }
+    return newcolor;
+}
+//END utility annotation functions
+
 }
