@@ -6,6 +6,21 @@
 //
 //========================================================================
 
+//========================================================================
+//
+// Modified under the Poppler project - http://poppler.freedesktop.org
+//
+// All changes made under the Poppler project to this file are licensed
+// under GPL version 2 or later
+//
+// Copyright (C) 2006 Dom Lachowicz <cinamod@hotmail.com>
+// Copyright (C) 2007-2008 Albert Astals Cid <aacid@kde.org>
+//
+// To see a description of the changes please see the Changelog file that
+// came with your tarball or type make ChangeLog if you are building from git
+//
+//========================================================================
+
 #include "config.h"
 #include <poppler-config.h>
 #include <stdio.h>
@@ -15,6 +30,7 @@
 #include <time.h>
 #include <math.h>
 #include "parseargs.h"
+#include "printencodings.h"
 #include "goo/GooString.h"
 #include "goo/gmem.h"
 #include "GlobalParams.h"
@@ -28,8 +44,9 @@
 #include "PDFDoc.h"
 #include "CharTypes.h"
 #include "UnicodeMap.h"
+#include "PDFDocEncoding.h"
 #include "Error.h"
-#include "UGooString.h"
+#include "DateInfo.h"
 
 static void printInfoString(Dict *infoDict, char *key, char *text,
 			    UnicodeMap *uMap);
@@ -43,11 +60,11 @@ static GBool printMetadata = gFalse;
 static char textEncName[128] = "";
 static char ownerPassword[33] = "\001";
 static char userPassword[33] = "\001";
-static char cfgFileName[256] = "";
 static GBool printVersion = gFalse;
 static GBool printHelp = gFalse;
+static GBool printEnc = gFalse;
 
-static ArgDesc argDesc[] = {
+static const ArgDesc argDesc[] = {
   {"-f",      argInt,      &firstPage,        0,
    "first page to convert"},
   {"-l",      argInt,      &lastPage,         0,
@@ -58,12 +75,12 @@ static ArgDesc argDesc[] = {
    "print the document metadata (XML)"},
   {"-enc",    argString,   textEncName,    sizeof(textEncName),
    "output text encoding name"},
+  {"-listenc",argFlag,     &printEnc,      0,
+   "list available encodings"},
   {"-opw",    argString,   ownerPassword,  sizeof(ownerPassword),
    "owner password (for encrypted files)"},
   {"-upw",    argString,   userPassword,   sizeof(userPassword),
    "user password (for encrypted files)"},
-  {"-cfg",        argString,      cfgFileName,    sizeof(cfgFileName),
-   "configuration file to use in place of .xpdfrc"},
   {"-v",      argFlag,     &printVersion,  0,
    "print copyright and version info"},
   {"-h",      argFlag,     &printHelp,     0,
@@ -97,18 +114,27 @@ int main(int argc, char *argv[]) {
 
   // parse args
   ok = parseArgs(argDesc, &argc, argv);
-  if (!ok || argc != 2 || printVersion || printHelp) {
-    fprintf(stderr, "pdfinfo version %s\n", xpdfVersion);
+  if (!ok || (argc != 2 && !printEnc) || printVersion || printHelp) {
+    fprintf(stderr, "pdfinfo version %s\n", PACKAGE_VERSION);
+    fprintf(stderr, "%s\n", popplerCopyright);
     fprintf(stderr, "%s\n", xpdfCopyright);
     if (!printVersion) {
       printUsage("pdfinfo", "<PDF-file>", argDesc);
     }
     goto err0;
   }
-  fileName = new GooString(argv[1]);
 
   // read config file
-  globalParams = new GlobalParams(cfgFileName);
+  globalParams = new GlobalParams();
+
+  if (printEnc) {
+    printEncodings();
+    delete globalParams;
+    goto err0;
+  }
+
+  fileName = new GooString(argv[1]);
+
   if (textEncName[0]) {
     globalParams->setTextEncoding(textEncName);
   }
@@ -201,8 +227,8 @@ int main(int argc, char *argv[]) {
 
   // print page size
   for (pg = firstPage; pg <= lastPage; ++pg) {
-    w = doc->getPageMediaWidth(pg);
-    h = doc->getPageMediaHeight(pg);
+    w = doc->getPageCropWidth(pg);
+    h = doc->getPageCropHeight(pg);
     if (multiPage) {
       printf("Page %4d size: %g x %g pts", pg, w, h);
     } else {
@@ -330,7 +356,7 @@ static void printInfoString(Dict *infoDict, char *key, char *text,
 	    (s1->getChar(i+1) & 0xff);
 	i += 2;
       } else {
-	u = s1->getChar(i) & 0xff;
+	u = pdfDocEncoding[s1->getChar(i) & 0xff];
 	++i;
       }
       n = uMap->mapUnicode(u, buf, sizeof(buf));
@@ -344,18 +370,16 @@ static void printInfoString(Dict *infoDict, char *key, char *text,
 static void printInfoDate(Dict *infoDict, char *key, char *text) {
   Object obj;
   char *s;
-  int year, mon, day, hour, min, sec;
+  int year, mon, day, hour, min, sec, tz_hour, tz_minute;
+  char tz;
   struct tm tmStruct;
   char buf[256];
 
   if (infoDict->lookup(key, &obj)->isString()) {
     fputs(text, stdout);
     s = obj.getString()->getCString();
-    if (s[0] == 'D' && s[1] == ':') {
-      s += 2;
-    }
-    if (sscanf(s, "%4d%2d%2d%2d%2d%2d",
-	       &year, &mon, &day, &hour, &min, &sec) == 6) {
+    // TODO do something with the timezone info
+    if ( parseDateString( s, &year, &mon, &day, &hour, &min, &sec, &tz, &tz_hour, &tz_minute ) ) {
       tmStruct.tm_year = year - 1900;
       tmStruct.tm_mon = mon - 1;
       tmStruct.tm_mday = day;
